#import "MenuFocusAppDelegate.h"

@implementation MenuFocusAppDelegate

@synthesize window;
@synthesize contextSelector;
@synthesize statusMenu;
@synthesize omniReference;

+ (void)initialize{
    NSMutableDictionary *defaultValues = [NSMutableDictionary dictionary];
    [defaultValues setObject:@"none" forKey:@"selectedContext"];
    [[NSUserDefaults standardUserDefaults] registerDefaults:defaultValues];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    OFApplication *omniFocus = [[OFApplication alloc] initWithBundleID: @"com.omnigroup.OmniFocus"];
    [self setOmniReference: [[omniFocus documents] at:1]];
    
    statusItem = [[[NSStatusBar systemStatusBar] statusItemWithLength:NSVariableStatusItemLength] retain];
    [statusItem setMenu:statusMenu];
    [statusItem setTitle:@"NextAction"];
    [statusItem setHighlightMode:YES];
    
    [[self statusMenu] removeAllItems];
    
    NSMenuItem *myItem = [[NSMenuItem alloc] initWithTitle:@"Please choose a context" action:@selector(void) keyEquivalent:@""];
    
    [[self statusMenu] addItem: myItem];
    [myItem release];
    
    [self addPreferenceOptionToMenu];
    
    NSArray *contexts = [self getContexts];
    
    [contextSelector removeAllItems];
    [contextSelector addItemsWithTitles: contexts];
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [contextSelector selectItemWithTitle:[defaults objectForKey:@"chosenContext"]];
    if ([defaults objectForKey:@"chosenContext"] != @"none"){
        [self updateTaskList:nil];
    }
}

- (NSString*) chosenContext{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    return [defaults objectForKey:@"chosenContext"];
}

- (void) setChosenContext:(NSString*) context{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:context forKey:@"chosenContext"];
}

- (IBAction) updateTaskList:(id)sender{

    NSString *selectedContext = [[[self contextSelector] selectedItem] title];
    [self setChosenContext:selectedContext];
    NSArray *actions = [self actionsForContext:selectedContext];
    
    [[self statusMenu] removeAllItems];
    [statusItem setTitle:selectedContext];
    for (int i=0; i<[actions count]; i++) {
        
        MFMenuItem *myItem = [[MFMenuItem alloc] initWithAction:[actions objectAtIndex:i] project:[actions objectAtIndex:i] andSelector:@selector(updateTaskList:)];
        [[myItem controller] setActionDone:YES];
        [[self statusMenu] addItem: myItem];
        [myItem release];
    }
    
    [self addPreferenceOptionToMenu];
}

- (NSArray*) getContexts{
    OFReference *of = [self omniReference];
    NSArray *contexts = [[[of flattenedContexts] name] getList];
    return contexts;
}

- (NSArray*) actionsForContext:(NSString*) context{
    OFReference *of = [self omniReference];
    NSString *selectedItem = context;
    NSArray *actions = 
    [[[[[of flattenedContexts] byName: selectedItem] availableTasks] name] getList];   
    return actions;
}

- (IBAction) showPreferences:(id)sender{
    [[self window] makeKeyAndOrderFront:self];
}

- (void) addPreferenceOptionToMenu{
    NSMenuItem *preferenceItem = [[NSMenuItem alloc] initWithTitle:@"Preferences" action:@selector(showPreferences:) keyEquivalent:@""];    
    NSMenuItem *quitItem = [[NSMenuItem alloc] initWithTitle:@"Quit" action:@selector(terminate:) keyEquivalent:@"q"];    
    [[self statusMenu] addItem: [NSMenuItem separatorItem]];
    [[self statusMenu] addItem: preferenceItem];
    [[self statusMenu] addItem: quitItem];
    [preferenceItem release];
    [quitItem release];
}

- (IBAction) markActionAsDone: (id)sender {
    NSMenuItem *action = sender;
    OFReference *of = [self omniReference];
    NSString *selectedContext = [[[self contextSelector] selectedItem] title];

    OFReference *chosenAction = 
    [[[[[of flattenedContexts] byName: selectedContext] availableTasks] byName: [action title]] getItem];
    [chosenAction complete];
}

@end
